/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 126987 $
 * \date $Date:: 2015-09-08 17:51:58 +0400#$
 * \author $Author: pav $
 * \brief   .
 */

#include "fkchdimgprj.h"

static TFKCHDImgContext * staticcardctx = NULL;

static TSupErr FirstInitCardCtx (TFKCHDImgContext * ctx) {
    BOOL isRelease;
    memset(&ctx->cont,0,sizeof(TOKEN_CONTAINER));
    ctx->cont.opened = TRUE;
    ctx->cont.C.c_dh = C_DH;
    ctx->cont.C.c_sig =C_SGN;
    ctx->cont.C.c_eke =C_EKE;
    ctx->cont.C.c_fls =C_FALSE;
    ctx->cont.C.c_seqfls =C_SEQ_FALSE;
    ctx->cont.abilityInitKeys = ABILITY_INIT_KEY;
    ctx->cont.abilityWriteKeys = ABILITY_WRITE_KEY;
    if(!CSPVersion(&isRelease)) 
	ctx->cont.abilityWriteKeys = !isRelease;
    ctx->cont.disablePass = FALSE;
    _tcscpy(ctx->cont.defaultPass,_TEXT("passw"));
    strcpy((char*)ctx->cont.pszKeyOID, szOID_GostR3410_2001_CryptoPro_XchA_ParamSet);
    return SUP_ERR_NO;
}

static DWORD makeProviderName (TCHAR ** resName)
{
    DWORD err = 0;
    TCHAR * szProvName = NULL;
    TSupErr code = 0;
    size_t len = 0;


    code = support_registry_get_string(_TEXT("\\Config\\KeyDevices\\FKCHDIMG\\Default\\destprov"),&len, szProvName);
    if (code)
    {
	err = code;
	goto done;
    } 
    szProvName = malloc ((len+1)*sizeof(TCHAR));
    if (!szProvName)
    {
	err = SUP_ERR_MEMORY;
	goto done;
    }
    code = support_registry_get_string(_TEXT("\\Config\\KeyDevices\\FKCHDIMG\\Default\\destprov"),&len, szProvName);
    /*, , CP_GR3410_2001_PROV. CSP3.6,  ,  KC1*/
    if (code)
    {
	free (szProvName);
	szProvName = NULL;
	err = code;
	goto done;

    } 
    *resName = szProvName;

done:
    return err;
}

/*!
 * \ingroup hdimage_fun_reader
 * \brief   .
 * \param context [in]  .
 * \param info [in/out]  #TSupSysInfoRegister
 * \sa #TSupSysInfoRegister, #SUPSYS_FUN_REGISTER, 
 *  #TSupSysFunction, #hdimage_info_unregister, 
 *  #rdr_unregister
 * \return  
 * \retval #SUP_ERR_NO  
 * \retval #SUP_ERR_PARAM    
 * \retval #SUP_ERR_MEMORY    
 * \retval #SUP_ERR_VERIFY     
 */
TSupErr fkchdimg_register(
    TSupSysContext *context, 
    TSupSysInfo *info )
{
    TSupSysInfoRegister *inf = (TSupSysInfoRegister*)info;
    TFKCHDImgContext *ctx;
    TSupErr code;

    SUPSYS_PRE( context == NULL );
    SUPSYS_PRE_INFO( info, TSupSysInfoRegister );

    inf->size_of = sizeof( TSupSysInfoRegister );
    if( support_load_library() )
	return SUP_ERR_VERIFY;
#ifndef UNIX
    if( support_path2dir( NULL, NULL, NULL ) )
	return SUP_ERR_VERIFY;
#endif
    inf->context = NULL;
    ctx = malloc( sizeof( *ctx ) );
    if( ctx == NULL )
    {
	support_unload_library();
	return SUP_ERR_MEMORY;
    }
    memset( ctx, 0, sizeof( *ctx ) );	// TODO:    
    ctx->path = NULL;
    code = hdimage_path_default(&( ctx->path ), 0 );
    if( code )
    {
	support_unload_library(); 
	free( ctx->path ); 
	free( ctx );
	return code;
    }
    ctx->folder = NULL;
    ctx->connect[0] = 0;

#ifdef UNIX
    ctx->handle = NULL;
#else /* UNIX */
    ctx->handle = INVALID_HANDLE_VALUE;
#endif /* UNIX */

////////////////////////////////////////////////////////////////
    if (staticcardctx ==NULL) {
	BOOL isRelease;
	staticcardctx = malloc( sizeof( *staticcardctx ) );
	memset(staticcardctx, 0, sizeof( *staticcardctx ) );	// TODO:    
	if( staticcardctx == NULL )
	{
	    support_unload_library();
	    free( ctx->path ); 
	    free (ctx);
	    return SUP_ERR_MEMORY;
	}
	staticcardctx->path = NULL;
	code = hdimage_path_default( &( staticcardctx->path ), 0 );
	if( code )
	{
    	    support_unload_library(); 
    	    free( staticcardctx->path ); 
	    free (staticcardctx);
	    free( ctx->path ); 
    	    free( ctx );
    	    return code;
        }
	staticcardctx->folder = NULL;
	staticcardctx->connect[0] = 0;
#ifdef UNIX
	staticcardctx->handle = NULL;
#else /* UNIX */
	staticcardctx->handle = INVALID_HANDLE_VALUE;
#endif /* UNIX */
	staticcardctx->folder = NULL;
	staticcardctx->cardctx = NULL;
	staticcardctx->file_name = NULL;
	staticcardctx->names.FriendlyName = NULL;
	code = create_lock (NULL, staticcardctx);
	if (code  != SUP_ERR_NO)
	    return code;
	code = fkchdimg_infofile_read (staticcardctx);
	if (code != SUP_ERR_NO) {
	    code = FirstInitCardCtx(staticcardctx);
	    if (code != SUP_ERR_NO) 
	    {
		free( staticcardctx->path ); 
		free (staticcardctx );
		free( ctx->path ); 
		free( ctx );
		return code;
	    }
	    code = fkchdimg_infofile_write (staticcardctx);
	    if (code != SUP_ERR_NO) 
	    {
		free( staticcardctx->path ); // 
		free (staticcardctx );
		free( ctx->path ); 
		free( ctx );
		return code;
	    }
	}
	code = makeProviderName (&staticcardctx->provname);
	if (code)   {
	    free( staticcardctx->path );
	    free (staticcardctx );
	    free( ctx->path ); 
	    free( ctx );
	    return code;
	}
	if(!CSPVersion(&isRelease)) 
	    staticcardctx->cont.abilityWriteKeys = !isRelease;
    }
    ctx->cardctx = staticcardctx;
    inf->context = ctx;
    return SUP_ERR_NO;
}

TSupErr fkchdimg_local_machine( 
    void *context, 
    TSupSysInfo *info )
{
    TReaderInfoLocalMachine *inf = (TReaderInfoLocalMachine*)info;
    TFKCHDImgContext *ctx = (TFKCHDImgContext*)context;
    TSupErr code;
    TCHAR *path;

    SUPSYS_PRE_CONTEXT( context, TFKCHDImgContext );
    SUPSYS_PRE_INFO( info, TReaderInfoLocalMachine );

    if( inf->flag & 1 )
    {
	code = hdimage_path_default( &path, 1 );
	ctx->machine=1;
    }
    else
    {
	code = hdimage_path_default( &path, 0 );
	ctx->machine=0;
    }
    if( code )
	return code;
    if( ctx->path )
	free( ctx->path );
    ctx->path = path;
    inf->size_of = sizeof( TReaderInfoLocalMachine );
    return SUP_ERR_NO;
}

TSupErr fkchdimg_unregister( 
    TSupSysContext *context, 
    TSupSysInfo *info )
{
    TFKCHDImgContext *ctx = (TFKCHDImgContext*)context;

    /*   . */
    SUPSYS_PRE_INFO( info, TSupSysInfoUnregister );
    if( ctx == NULL )
	return SUP_ERR_NO;
    free(ctx->path);
    free(ctx);

////////////////////////////////////////////////////////////////

    support_unload_library();
    return SUP_ERR_NO;
}

BOOL CSPVersion(BOOL *isrelease) 
{
    HCRYPTPROV prov;
    BOOL err = FALSE;
    TSupErr code = 0;
    long is_able;
    PROV_PP_VERSION_EX ver;
    DWORD length;

    code = support_registry_get_long(_TEXT("\\Config\\KeyDevices\\FKCHDIMG\\Default\\canset"),&is_able);
    if (code || is_able == 0)
    {
	*isrelease = TRUE;
	return FALSE;
    } 

    if (!CryptAcquireContext(&prov,NULL,staticcardctx->provname,PROV_GOST_2001_DH,CRYPT_VERIFYCONTEXT))
	return TRUE;
    if (!CryptGetProvParam(prov,PP_VERSION_EX,NULL,&length,0)) {
	err = TRUE;
	goto done;
    }
    if (length != (DWORD)sizeof(PROV_PP_VERSION_EX)) {
	err = TRUE;
	goto done;
    }
    if (!CryptGetProvParam(prov,PP_VERSION_EX,(BYTE*)&ver,&length,0)) {
	err = TRUE;
	goto done;
    }
    *isrelease = (ver.TypeDebRel == VER_TYPE_RELEASE);
done:
    CryptReleaseContext(prov, 0);
    return err;
}
